"use strict";
this.name               = "navi_mfd";
this.author               = "spara";
this.copyright            = "2014 Spara";
this.description         = "Show navigational info in an MFD";
this.version            = "1.0.1";
this.licence     = "CC BY-NC-SA 3.0";

//1.0.1 Tracker oxp handling.
//1.0.1 Bug fix in current target id.
	
this.startUp = function() {
	//User options start here
	this.$interval = 1; //Refresh frequency. (in secs)
	this.$distUnit = "OU"; //Distance unit to show.
	this.$permanentBasis = true; //True will define the basis of the distance unit at the first run and keep it permanent over saves. False will define the basis on entering a system.
	this.$distUnitBasis = 2; //The basis of the distance unit. 1 for kms, 2 for planet-sun, 3 for wp-planet
		
	//initialize dist unit basis
	if (this.$permanentBasis) {
		if (!missionVariables.aadHudUnit) {
			this.$defineUnitBasis();
			missionVariables.aadHudUnit = this.$ostronomicalUnit;
		}
		else this.$ostronomicalUnit = missionVariables.aadHudUnit;
	}
	else this.$defineUnitBasis();
	
	//timer for display update
	this.$distanceTimer = new Timer(this,this.$updateDistance,0,this.$interval);
	this.$distanceTimer.stop();
	
	//storage variable
	this.$targetName = "unknown";
	
	//symbols
	this.$govs = new Array();
	for (var i = 0; i < 8 ; i++)
		this.$govs.push(String.fromCharCode(i));
	this.$ecos = new Array();
	for (i = 0; i < 8 ; i++)
		this.$ecos.push(String.fromCharCode(23 - i));
}

//define unit basis
this.$defineUnitBasis = function() {
	switch(this.$distUnitBasis) {
		case 1: //distance in kms
			this.$ostronomicalUnit = 1000;
			break;
		case 2: //distance based on planet-sun distance
			this.$ostronomicalUnit = system.sun.position.distanceTo(system.mainPlanet);
			break;
		case 3: //distance base on planet-wp distance
			this.$ostronomicalUnit = system.mainPlanet.position.magnitude();
			break;
	}
}

//update hyperspace display data
this.shipWillLaunchFromStation = this.shipExitedWitchspace = function() {
	this.$setHyperMessage();
}

//update hyperspace display data
this.guiScreenChanged = function(to, from) {
	if (from === "GUI_SCREEN_SHORT_RANGE_CHART")
		this.$setHyperMessage();
		this.$updateDistance();
}

//update compass target data and start timer
this.shipLaunchedFromStation = function() {
	this.compassTargetChanged(player.ship.compassTarget, player.ship.compassMode);
	this.$distanceTimer.start();
}

//stop timer
this.shipWillDockWithStation = function() {
	this.$distanceTimer.stop();
}

//update mfd
this.$updateDistance = function() {
	player.ship.setMultiFunctionText("navi_mfd",this.$message());
}

//compose output
this.$message = function() {
	//hyperspace data is updated only when it changes
	var message = this.$hyperMessage;
	message = message + "Hyperdrive Status:\n" + this.$status();
	var target = player.ship.compassTarget;
	//valid target case
	if (target) {
		message = message + "Space Compass Target:\n" + this.$truncate("  " + this.$targetName, 14) + "\n";
		var distInM = player.ship.position.distanceTo(target) - target.collisionRadius;
		if (distInM < 0) distInM = 0;
		var dist = distInM/this.$ostronomicalUnit;
		message = message + "Distance: " + dist.toFixed(6) + " " + this.$distUnit +"\n";
		message = message + "Estimated Travel Time: ";
		var vPtoT = target.position.subtract(player.ship.position); //player-target vector
		if (player.ship.velocity.dot(vPtoT) > 0.1) {//check heading
			var  vVelPtoT = vPtoT.multiply(player.ship.velocity.dot(vPtoT) / (vPtoT.dot(vPtoT))); //velocity vector projection on player-target vector    
			var timeToTarget = distInM / vVelPtoT.magnitude();
			var hours = Math.floor(timeToTarget / 3600);
			var mins = Math.floor((timeToTarget - 3600 * hours) / 60);
			var secs = Math.floor(timeToTarget - hours * 3600 - mins * 60);
			if (hours < 10) 
				hours = "0"+hours;
			if (mins < 10) 
				mins = "0"+mins;
			if (secs < 10) 
				secs = "0"+secs;
			message = message + hours+":"+mins+":"+secs;
		}
		else message = message + "**:**:**";
	}
	//null case
	else {
		message = message + "Space Compass Target:\n  -\n";
		message = message + "Distance: -\n";
		message = message + "Estimated Travel Time: **:**:**";
	}
	return message;
}

//compose hyperspace output
this.$setHyperMessage = function() {
	//no hyperdrive
	if (!player.ship.hasHyperspaceMotor) {
		var hyperSystem = "-";
		var hyperTime = "**:**:**\n";
	}
	else {
		var hyperSystem = System.infoForSystem(galaxyNumber, player.ship.targetSystem).name;
		this.$hyperDistance = System.infoForSystem(galaxyNumber, player.ship.targetSystem).distanceToSystem(system.info);
		//no target set
		if (hyperSystem === system.name) {
			var hyperSystem = "-";
			var hyperTime = "**:**:**\n";
		}
		else if (this.$hyperDistance > 7) {
			var hyperTime = "**:**:**\n";
		}
		else {
			var hyperTime = this.$hyperDistance * this.$hyperDistance;//in hours
			var hours = Math.floor(hyperTime);
			var mins = Math.floor((hyperTime - hours) * 60);
			var secs = Math.floor((hyperTime - hours - mins / 60) * 3600);
			if (hours < 10) 
				hours = "0"+hours;
			if (mins < 10) 
				mins = "0"+mins;
			if (secs < 10) 
				secs = "0"+secs;
			hyperTime = hours+":"+mins+":"+secs+"\n";
		}
	}
	if (system.isInterstellarSpace)
		this.$hyperMessage = "Present System: " + system.name + "\n";
	else
		//present system with gov symbol
		this.$hyperMessage = this.$hyperSystemLine("Present System:", system.name, this.$govs[system.government])+"\n";
		//present system with eco + gov symbols
		//this.$hyperMessage = this.$hyperSystemLine("Present System:", system.name, this.$ecos[system.economy] + this.$govs[system.government])+"\n";

	//hyper system, no symbol
	this.$hyperMessage = this.$hyperMessage + "Hyperspace System: " + hyperSystem + "\n";
		
	//hyper system, eco + gov symbol
	/*
	if (hyperSystem !== "-")
		this.$hyperMessage = this.$hyperMessage + this.$hyperSystemLine("Hyperspace System:", hyperSystem, this.$ecos[System.infoForSystem(galaxyNumber, player.ship.targetSystem).economy] + this.$govs[System.infoForSystem(galaxyNumber, player.ship.targetSystem).government]) + "\n";
	else
		this.$hyperMessage = this.$hyperMessage + "Hyperspace System: " + hyperSystem + "\n";
	*/
	
	this.$hyperMessage = this.$hyperMessage + "Estimated Travel Time: " + hyperTime;
}

this.$hyperSystemLine = function(preText, systemName, systemSymbol) {
	var line = preText + " " + systemName;
	if (systemName === "-") return line;
	//space it up
	while (global.defaultFont.measureString(line + systemSymbol) < 14)
		line = line + " ";
	while (global.defaultFont.measureString(line + systemSymbol) < 15)
		line = line + String.fromCharCode(31);
	return line + systemSymbol;
}

//update compass target name. there's some cim still here.
this.compassTargetChanged = function(whom, mode) {
	this.$targetName = "Initializing...";
	if (!whom) return;//null case
	if (whom.displayName) this.$targetName = whom.displayName;
	else this.$targetName = whom.name;
	if (mode === "COMPASS_MODE_BASIC") {
		if (whom.isPlanet) this.$targetName = "Planet";
		else this.$targetName = "Main Station";
	}
	else if (mode === "COMPASS_MODE_BEACONS") {
		// Escape capsule locator OXP
		if (this.$targetName === "Metal fragment" && whom.beaconCode.substr(0,1) === "E")
			this.$targetName = "Escape Capsule";
		else if (this.$targetName === "Navigation Buoy") {
			if (whom.beaconCode.substr(0,1) === "g")
				this.$targetName = "GRS Buoy Factory Nav Buoy";
			else if (whom.beaconCode.substr(0,1) === "N")
				this.$targetName = "Main Station Nav Buoy";
		}
		// Tracker OXP
		else if (this.$targetName === "Tracker") {
			if (whom.target.displayName)
				this.$targetName = whom.target.displayName;
			else this.$targetName = whom.target.name;
			this.$targetName = this.$targetName + " (Tracked)";
		}
	}
	else if (mode === "COMPASS_MODE_PLANET")
		this.$targetName = "Planet";
	else if (mode === "COMPASS_MODE_SUN") {
		//distant suns oxp
		if (system.info.sun_name)
			this.$targetName = system.info.sun_name + " (Sun)";
		else this.$targetName = "Sun";
	}
	else if (mode === "COMPASS_MODE_STATION")
		this.$targetName = "Main Station";
	this.$updateDistance();
}

//hyperdrive status. mass locks, fuel etc. there's some cim here too.
this.$status = function() {
	if (!player.ship.hasHyperspaceMotor)
		return "  Not installed\n\n";
	if (this.$hyperDistance === 0) return "  No target system set\n\n";
	if (this.$hyperDistance > 7) return "  Target out of range\n\n";
	if (player.ship.fuel < this.$hyperDistance)
		return "  Not enough fuel\n\n";
	var position = player.ship.position;
	function isBlocker(entity) {
		if (!entity.isShip) return false;
		if (entity.mass / position.squaredDistanceTo(entity) >= 10)
			return true;
		return false;
	}
	var blockers = system.filteredEntities(this, isBlocker, player.ship, 25600);
	if (blockers.length > 0)
		return this.$truncate("  Too close to " + blockers[0].name, 14) + "\n\n";
	return "  Ready\n\n";
}

//squeezing looks ugly, so I'll truncate instead.
this.$truncate = function(line, width) {
	var lastChar = "";
	//remove characters from the end of the line.
	while (global.defaultFont.measureString(line) > width) {
		lastChar = line.charAt(line.length - 1);
		line = line.slice(0, -1);
	}
	//no chopping was done. return
	if (lastChar === "") return line;
	//if the last char is a space or last removed char was space, return
	if (line.charAt(line.length - 1) === " " || lastChar === " ") {
		return line;
	}
	//if the last char is a char and the second last is a space, remove the char and return
	if (line.charAt(line.length - 2) === " ") {
		return line.slice(0, -1);
	}
	//it seems that we have a chopped word in our hands. looks nicer if we end it with a dot
	line = line + ".";
	while (global.defaultFont.measureString(line) > width) {
		line = line.slice(0, -2);
		line = line + ".";
	}
	return line;
}
